package info.aspirecn.rdc.iov.sjjh.servicenode.supplier.jk.action;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Preconditions;
import info.aspirecn.iov.sjjh.commons.lang.ChannelTypeHandleResponseObject;
import info.aspirecn.iov.sjjh.commons.lang.Constant;
import info.aspirecn.rdc.aspirecloud.node.except.utils.ErrorUtils;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.jk.service.JKChannelService;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.jk.vo.TwoParamIdCardResponseObject;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.jk_interface.action.JKChannelAction;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.jk_interface.vo.EncryptionMobileCheckResponseObject;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.jk_interface.vo.EncryptionResponseObject;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.jk_interface.vo.ResponseObject;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.tomcat.util.ExceptionUtils;
import org.apache.tomcat.util.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;


/**
 *
 * @author jianggang
 * @Date 2018/10/24 14:36:00
 * @Description 交科供应商-上游接口调用controller
 */

@RestController
@Slf4j
public class JKChannelActionImp implements JKChannelAction {

	@Autowired
    private ObjectMapper objectMapper;
    @Autowired
    JKChannelService jkChannelService;

	/**日志参数中的错误参数名*/
	private static final String LOG_ERROR_PARA = "x-responseCode";
	/**日志参数中的是否收费*/
	private static final String LOG_FEE_PARA = "x-isCharge";

    @Override
    public ResponseObject TwoParamIdCard(
            @RequestHeader(name = "channelId",required = true) String channelId,
            @RequestParam(name = "outTime", required = false) Integer outTime,
            @RequestParam(name = "customBody",required = true) String customBody) {
        log.info("交科身份证而要素请求--->channelId:{}--->outTime:{},--->customBody:{}",channelId,outTime,customBody);

        ResponseObject responseObject = jkChannelService.sendTwoParamIdCardRequest(channelId,outTime,customBody);
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes)RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        //日志记录通道类型-同步
        request.setAttribute(Constant.CHANNEL_TYPE_KEY,Constant.CHANNEL_TYPE_SYNC);
		//日志记录通道响应码
        if(responseObject.getCode() != 1){
            request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_ERROR_CODE);
        }else{
            request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_SUCCESS_CODE);
        }
        //日志记录通道请求参数
        if(customBody != null){
            try {
                request.setAttribute(Constant.CHANNEL_LOG_QUERY,Base64.encodeBase64String(customBody.getBytes("utf-8")));
            } catch (UnsupportedEncodingException e) {
                ExceptionUtils.handleThrowable(e);
                log.error("异常信息{}",e);
            }
        }
        Object resultBody = responseObject.getResultBody();
        //日志记录上游接口响应码
        if(resultBody != null){
        	Integer upStreamCode = ((TwoParamIdCardResponseObject)responseObject.getResultBody()).getCode();
            request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE,String.valueOf(upStreamCode));
        }else{
            request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE,Constant.LOG_UPSTREAM_DEFAULT_RESPONSE_CODE);
        }
        request.setAttribute("x-isCharge",responseObject.getIsCharge());
        return responseObject;
    }

    @Override
	@ApiOperation(value = "移动三要素加密版接口", notes = "")
    public EncryptionResponseObject encryptionMobileCheck(
    		@ApiParam(value="通道ID")@RequestHeader(name="channelId")  String channelId,
    		@ApiParam(value="超时时间,单位：毫秒",example = "10000")@RequestParam(name = "outTime", required = true) int outTime,
    		@ApiParam(value="请求参数JSON串")@RequestParam(name = "customBody", required = true) String customBody) {
		Preconditions.checkNotNull(channelId, "通道ID为空");
		Preconditions.checkNotNull(customBody, "三要素加密版接口请求参数为空");

		HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes())
				.getRequest();
		EncryptionResponseObject ret = jkChannelService.encryptionMobileCheck(customBody, outTime);

		if(StringUtils.isBlank(channelId)) {
			ret.setChannelId(Constant.JK_ENCRYPTION_CHANNEL_ID);
		} else {
			ret.setChannelId(channelId);
		}

        //把接口参数、调用结果和是否收费放入访问日志中
        if(ret.getCode() != Constant.SUCCESS) {
    		request.setAttribute(LOG_ERROR_PARA, Constant.CHANNEL_LOG_ERROR_CODE);
    	} else {
    		request.setAttribute(LOG_ERROR_PARA, Constant.CHANNEL_LOG_SUCCESS_CODE);
    	}

		String upstreamCode = ""+Constant.LOG_UPSTREAM_DEFAULT_RESPONSE_CODE;
		try {
	    	request.setAttribute(LOG_FEE_PARA, ret.getIsCharge());
    		request.setAttribute(Constant.CHANNEL_LOG_QUERY,
    				Base64.encodeBase64String(customBody.getBytes("UTF-8")));

    		//解析返回，获取接口返回的code
	    	String jsonStr = objectMapper.writeValueAsString(ret.getResultBody());
	    	EncryptionMobileCheckResponseObject responseObject = objectMapper.readValue(jsonStr,
	    			EncryptionMobileCheckResponseObject.class);
	    	if(responseObject != null) {
	    		upstreamCode = responseObject.getCode();
	    	}
    	} catch (Exception ex) {
    		ErrorUtils.captureException(ex);
    	}
		request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, upstreamCode);

        return ret;
	}
    @Override
   	@ApiOperation(value = "移动三要素加密版接口", notes = "")
       public ChannelTypeHandleResponseObject encryptionMobileCheckNew(
       		@ApiParam(value="通道ID")@RequestHeader(name="channelId")  String channelId,
       		@ApiParam(value="超时时间,单位：毫秒",example = "10000")@RequestParam(name = "outTime", required = true) int outTime,
       		@ApiParam(value="请求参数JSON串")@RequestParam(name = "customBody", required = true) String customBody) {
    	log.info("channelId:{},outTime:{},customBody:{}",channelId,outTime,customBody);
    	ChannelTypeHandleResponseObject channelType = new ChannelTypeHandleResponseObject();
        channelType.setCode(1);
        channelType.setIsCharge(1);
        channelType.setResultCode(1);
        channelType.setResultBody("匹配");
    	return channelType;
    }

	@Override
	public ChannelTypeHandleResponseObject vehicleMortgage(String channelId, int outTime, String customBody) {
		HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes())
				.getRequest();

		//调用service
		ChannelTypeHandleResponseObject responseObject = jkChannelService.vehicleMortgage(request, customBody, outTime);

		//把接口参数、调用结果和是否收费放入访问日志中
		if(responseObject.getCode() != Constant.SUCCESS) {
			request.setAttribute(LOG_ERROR_PARA, Constant.CHANNEL_LOG_ERROR_CODE);
		} else {
			request.setAttribute(LOG_ERROR_PARA, Constant.CHANNEL_LOG_SUCCESS_CODE);
		}
		request.setAttribute(LOG_FEE_PARA, responseObject.getIsCharge());

		try {
			request.setAttribute(Constant.CHANNEL_LOG_QUERY,
					Base64.encodeBase64String(customBody.getBytes("UTF-8")));
		} catch (Exception ex) {
			ErrorUtils.captureException(ex);
		}

		request.setAttribute(Constant.CHANNEL_TYPE_KEY, Constant.CHANNEL_TYPE_SYNC);
		return responseObject;
	}

	/**
	 * 参数校验
	 * @param channelId
	 * @param staticChannelId
	 * @return
	 */
	public EncryptionResponseObject checkParams(String channelId,String staticChannelId) {
		EncryptionResponseObject ret = new EncryptionResponseObject();
		if(!staticChannelId.equals(channelId)) {
			ret.setCode(Constant.CHANNEL_ID_NO_MATCH);
	        ret.setIsCharge(Constant.INTERFACE_QUERY_NO_FEE);ret.setMessage("channelid不匹配");

		}

		return ret;
	}
}
